@@ -3,5 +3,5 @@
 from toblerone import projection, pvestimation
 from toblerone.classes import ImageSpace, Surface, Hemisphere
 from toblerone._version import __version__
-
-__all__ = ['core', 'classes', 'utils']
+from toblerone.projection import Projector
+from toblerone import utils
\ No newline at end of file
