@@ -44,7 +44,7 @@ class ImageSpace(BaseSpace):
 
 
     @classmethod
-    def minimal_enclosing(cls, surfs, reference, affine):
+    def minimal_enclosing(cls, surfs, reference):
         """
         Return the minimal space required to enclose a set of surfaces. 
         This space will be based upon the reference, sharing its voxel 
@@ -55,8 +55,6 @@ class ImageSpace(BaseSpace):
         Args: 
             surfs: singular or list of surface objects 
             reference: ImageSpace object or path to image to use 
-            affine: np.array transformation INTO the reference space, 
-                in world-world mm terms (not FLIRT convention)
 
         Returns: 
             ImageSpace object, with a shifted origin and potentially different
@@ -74,26 +72,21 @@ class ImageSpace(BaseSpace):
         else: 
             space = copy.deepcopy(reference)
 
-        if affine is not None: 
-            overall = space.world2vox @ affine
-        else: 
-            overall = space.world2vox
-
         # Extract min and max vox coords in the reference space 
         min_max = np.empty((2*len(slist), 3))
         for sidx,s in enumerate(slist):
-            ps = utils.affineTransformPoints(s.points, overall)
+            ps = utils.affine_transform(s.points, space.world2vox)
             min_max[sidx*2,:] = ps.min(0)
             min_max[sidx*2 + 1,:] = ps.max(0)
 
         # Fix the offset relative to reference and minimal size 
-        minFoV = min_max.min(0).round().astype(np.int16)
-        maxFoV = min_max.max(0).round().astype(np.int16)
+        minFoV = np.floor(min_max.min(0)).astype(np.int16)
+        maxFoV = np.ceil(min_max.max(0)).astype(np.int16)
         size = maxFoV - minFoV + 1
         FoVoffset = -minFoV
     
         # Get a copy of the corresponding mm coords for checking later
-        min_max_mm = utils.affineTransformPoints(np.array([minFoV, maxFoV]),
+        min_max_mm = utils.affine_transform(np.array([minFoV, maxFoV]),
             space.vox2world)
 
         # Calculate new origin for the coordinate system and modify the 
@@ -102,7 +95,7 @@ class ImageSpace(BaseSpace):
         space.vox2world[0:3,3] = min_max_mm[0,:]
         space.offset = FoVoffset 
 
-        check = utils.affineTransformPoints(min_max_mm, space.world2vox)
+        check = utils.affine_transform(min_max_mm, space.world2vox)
         if (np.any(check[0,:].round() < 0) or 
             np.any(check[1,:].round() > size - 1)): 
             raise RuntimeError("New space does not enclose surfaces")
