@@ -11,7 +11,7 @@
 
 import functools
 import itertools
-import multiprocessing as mp 
+import multiprocessing as mp
 from scipy import sparse
 import copy 
 
@@ -25,16 +25,18 @@ from toblerone.ctoblerone import (_ctestTriangleVoxelIntersection,
                                   _cytestManyRayTriangleIntersections,
                                   _quick_cross)
 from toblerone import utils 
-from toblerone.utils import NP_FLOAT
+from toblerone.utils import NP_FLOAT, space_encloses_surface
 
 
 # Module level constants ------------------------------------------------------
 
+ZERO_3 = np.zeros(3, dtype=NP_FLOAT)
+
 # edge vectors for a single voxel 
 ORIGINS = np.array([1, 1, 1, 4, 4, 4, 5, 5, 8, 8, 6, 7, 1, 2, 3, 4,
-    1, 1, 1, 8, 8, 8, 2, 2, 3, 4, 4, 6], dtype=np.int8) - 1
+    1, 1, 1, 8, 8, 8, 2, 2, 3, 4, 4, 6], dtype=np.int32) - 1
 ENDS = np.array([2, 3, 5, 8, 2, 3, 6, 7, 6, 7, 2, 3, 8, 7, 6, 5,
-    6, 4, 7, 5, 3, 2, 3, 5, 5, 6, 7, 7], dtype=np.int8) - 1
+    6, 4, 7, 5, 3, 2, 3, 5, 5, 6, 7, 7], dtype=np.int32) - 1
 
 # iterating over dimensions xyz, xyz 
 DIMS = np.array([0,1,2,0,1,2])
@@ -56,12 +58,12 @@ SUBVOXCORNERS = (np.array([
 TETRA1 = np.array([[0,3,4,5],   # aABC
                    [0,1,2,4],   # abcB
                    [0,2,4,5]],  # acBC
-                   dtype=np.int8)  
+                   dtype=np.int32)  
 
 TETRA2 = np.array([[0,3,4,5],   # aABC
                    [0,1,2,5],   # abcC
                    [0,1,4,5]],  # abBC
-                   dtype=np.int8) 
+                   dtype=np.int32) 
 
 # For defining the edges of triangle within a mesh
 TRI_EDGE_INDEXING = [{1,0}, {2,0}, {2,1}]
@@ -149,6 +151,39 @@ def _separatePointClouds(tris):
     return groups 
 
 
+def form_associations(points_vox, tris, space, cores=mp.cpu_count()):
+    """
+    Identify which triangles of a surface intersect each voxel. This 
+    reduces the number of operations that need be performed later. The 
+    results will be stored on the surface object (ie, self)
+
+    Returns: 
+        None, but associations (sparse CSR matrix of size (voxs, tris)
+        and assocs_keys (array of voxel indices containint the surface)
+        will be set on the calling object. 
+    """
+
+    assert utils.space_encloses_surface(space, points_vox)
+    workerFunc = functools.partial(_formAssociationsWorker, 
+                                    tris, points_vox, space.size)
+
+    if cores > 1:
+        chunks = utils._distributeObjects(range(tris.shape[0]), cores)
+        with mp.Pool(cores) as p:
+            worker_assocs = p.map(workerFunc, chunks, chunksize=1)
+
+        assocs = worker_assocs[0]
+        for a in worker_assocs[1:]:
+            assocs += a 
+
+    else:
+        assocs = workerFunc(range(tris.shape[0]))
+
+    # Assocs keys is a list of all voxels touched by any triangle
+    assocs_keys = np.flatnonzero(assocs.sum(1).A)
+    return assocs, assocs_keys
+
+
 def _formAssociationsWorker(tris, points, grid_size, triInds):
     """
     Worker function for use with multiprocessing. See formAssociations
@@ -158,7 +193,7 @@ def _formAssociationsWorker(tris, points, grid_size, triInds):
     """
 
     vox_size = np.array([0.5, 0.5, 0.5], dtype=NP_FLOAT)
-    assocs = sparse.dok_matrix((grid_size.prod(), tris.shape[0]), dtype=np.bool)
+    assocs = sparse.dok_matrix((grid_size.prod(), tris.shape[0]), dtype=bool)
 
     for t in triInds:
 
@@ -205,7 +240,7 @@ def _findRayTriangleIntersections2D(testPnt, patch, axis):
     # And find the multipliers for those that do intersect 
     if np.any(fltr):
         mus = _findRayTriPlaneIntersections(patch.points[patch.tris[fltr,0],:],
-            patch.xProds[fltr,:], testPnt, ray)
+            patch.xprods[fltr,:], testPnt, ray)
     else:
         mus = np.array([])
 
@@ -230,10 +265,10 @@ def _findRayTriPlaneIntersections(planePoints, normals, testPnt, ray):
 
     # mu is defined as dot((p_plane - p_test), normal_tri_plane) ...
     #   / dot(ray, normal_tri_plane)
-    dotRN = (normals * ray).sum(1)
-    mu = ((planePoints - testPnt) * normals).sum(1) / dotRN 
+    dotRN = np.einsum('ij,j->i', normals, ray, casting='no')
+    mu = np.einsum('ij,ij->i', planePoints - testPnt, normals, casting='no')
 
-    return mu 
+    return mu / dotRN
 
 
 def _findRayTriangleIntersections3D(testPnt, ray, patch):
@@ -271,23 +306,25 @@ def _findRayTriangleIntersections3D(testPnt, ray, patch):
     # Calculate the projection of each point onto the direction vector of the
     # surface normal. Then subtract this component off each to leave their position
     # on the plane and shift coordinates so the test point is the origin.
-    lmbda = (patch.points * ray).sum(1)
-    onPlane = (patch.points - (lmbda[:,None] * ray[None,:])) - testPnt 
+    lmbda = np.einsum('ij,j->i', patch.points, ray, casting='no')
+    onPlane = (patch.points 
+                - np.einsum('i,j->ij', lmbda, ray, casting='no')
+                - testPnt)
 
     # Re-express the points in 2d planar coordiantes by evaluating dot products with
     # the d2 and d3 in-plane orthonormal unit vectors
-    onPlane2d = np.array([(onPlane * d1).sum(1), (onPlane * d2).sum(1),
-        np.zeros(lmbda.size)], dtype=NP_FLOAT)
+    onPlane2d = np.array([ np.einsum('ij,j->i', onPlane, d1, casting='no'),
+                           np.einsum('ij,j->i', onPlane, d2, casting='no'),
+                           np.zeros(lmbda.size, dtype=NP_FLOAT)])
 
     # Now perform the test 
-    start = np.zeros(3, dtype=NP_FLOAT)
-    fltr = _cytestManyRayTriangleIntersections(patch.tris, onPlane2d.T, start,
+    fltr = _cytestManyRayTriangleIntersections(patch.tris, onPlane2d.T, ZERO_3,
         0, 1)
 
     # For those trianglest that passed, calculate multiplier to point of 
     # intersection
     mus = _findRayTriPlaneIntersections(patch.points[patch.tris[fltr,0],:], 
-        patch.xProds[fltr,:], testPnt, ray)
+        patch.xprods[fltr,:], testPnt, ray)
     
     return mus
 
@@ -491,7 +528,7 @@ def _findVoxelSurfaceIntersections(patch, vertices):
                 fold = True
                 return (intersects, fold)
 
-            intPnts = pnt + (intMus[:,None] * edge[None,:])
+            intPnts = pnt + np.einsum('i,j->ij', intMus, edge, casting='no')
             intersects = np.vstack((intersects, intPnts[accept,:]))
 
     return (intersects, fold)
@@ -503,14 +540,14 @@ def _safeFormHull(points):
     elsewhere). For everything else, let the exception continue up. 
     """
 
-    if points.size > 3:
+    if points.size > 8:
         try:
             hull = ConvexHull(points)
             return hull.volume
         except QhullError: 
             return 0
-        except Exception: 
-            raise 
+        except Exception as e: 
+            raise e
     else: 
         return 0
         
@@ -556,9 +593,6 @@ def _estimateVoxelFraction(surf, voxIJK, voxIdx, supersampler):
 
     verbose = False
     # print(voxIdx)
-
-    # Hardcode voxel size as we now work in voxel coords. Intialise results
-    vox_size = np.array([1,1,1], dtype=np.int8)
     inFraction = 0.0
 
     # Set up the subvoxel sizes and vols. 
@@ -566,7 +600,7 @@ def _estimateVoxelFraction(surf, voxIJK, voxIdx, supersampler):
     subVoxVol = np.prod(subvox_size).astype(NP_FLOAT)
 
     # Rebase triangles and points for this voxel
-    voxCentFlag = surf.voxelised[voxIdx]
+    voxCentFlag = surf.indexed.voxelised[voxIdx]
     patch = surf.to_patch(voxIdx)
 
     # Test all subvox centres now and store the results for later
@@ -710,13 +744,12 @@ def _estimateFractions(surf, supersampler, descriptor, cores):
         vector of size prod(FoV)
     """
 
-    size = surf._index_space.size 
-
     supersampler = np.squeeze(np.array(supersampler, dtype=np.int16))
 
     # Compute all voxel centres, prepare a partial function application for 
     # use with the parallel pool map function 
-    workerChunks = utils._distributeObjects(range(surf.assocs_keys.size), 60)
+    workerChunks = utils._distributeObjects(
+                                range(surf.indexed.assocs_keys.size), 50)
     estimatePartial = functools.partial(_estimateFractionsWorker, 
         surf, supersampler)
 
@@ -759,8 +792,8 @@ def _estimateFractionsWorker(surf, supersampler, chunk):
     # pool the exception will not be raised)
     try:
         pvs = np.zeros(len(chunk), dtype=NP_FLOAT)
-        vox_inds = surf.assocs_keys[chunk]
-        vox_ijks = np.array(np.unravel_index(vox_inds, surf._index_space.size),
+        vox_inds = surf.indexed.assocs_keys[chunk]
+        vox_ijks = np.array(np.unravel_index(vox_inds, surf.indexed.space.size),
             dtype=NP_FLOAT).T
 
         for idx in range(len(chunk)):
@@ -785,10 +818,10 @@ def _voxelise_worker(surf, dim_range, raysd1d2):
             voxel grid, recording the D1 D2 coordinates of their origin
     """
 
-    size = surf._index_space.size
+    size = surf.indexed.space.size
     dim = np.argmax(size)
     other_dims = list({0,1,2} - {dim})
-    mask_size = copy.copy(size)
+    mask_size = copy.deepcopy(size)
     mask_size[other_dims[0]] = len(dim_range)
     mask = np.zeros(mask_size.prod(), dtype=bool)
 
@@ -848,8 +881,8 @@ def _voxelise_worker(surf, dim_range, raysd1d2):
         return mask.reshape(mask_size)
 
 
-def vox_tri_weights(in_surf, out_surf, spc, factor=10, cores=mp.cpu_count(), 
-                     ones=False):     
+def vox_tri_weights(in_surf, out_surf, spc, factor, 
+                    cores=mp.cpu_count(), ones=False):     
     """
     Form matrix of size (n_vox x n_tris), in which element (I,J) is the 
     fraction of samples from voxel I that are in triangle prism J. 
@@ -868,9 +901,15 @@ def vox_tri_weights(in_surf, out_surf, spc, factor=10, cores=mp.cpu_count(),
             NB this matrix is not normalised in any way!
     """
 
+    points_vox = []
+    for s in [in_surf, out_surf]:
+        p_v = utils.affine_transform(s.points, spc.world2vox)   
+        points_vox.append(p_v)
+
     n_tris = in_surf.tris.shape[0]
-    worker = functools.partial(_vox_tri_weights_worker, in_surf=in_surf, 
-        out_surf=out_surf, spc=spc, factor=factor, ones=ones)
+    worker = functools.partial(_vox_tri_weights_worker, 
+        inps_vox=points_vox[0], outps_vox=points_vox[1], 
+        tris=in_surf.tris, spc=spc, factor=factor, ones=ones)
     
     if cores > 1: 
         t_ranges = utils._distributeObjects(range(n_tris), cores)
@@ -884,10 +923,11 @@ def vox_tri_weights(in_surf, out_surf, spc, factor=10, cores=mp.cpu_count(),
     else: 
         vpmat = worker(range(n_tris))
          
-    return vpmat / (factor ** 3)
+    return vpmat / factor.prod()
 
 
-def _vox_tri_weights_worker(t_range, in_surf, out_surf, spc, factor, ones=False):
+def _vox_tri_weights_worker(t_range, inps_vox, outps_vox, tris, 
+                            spc, factor, ones=False):
     """
     Helper method for vox_tri_weights(). 
 
@@ -905,9 +945,10 @@ def _vox_tri_weights_worker(t_range, in_surf, out_surf, spc, factor, ones=False)
     # Initialise a grid of sample points, sized by (factor) in each dimension. 
     # We then shift the samples into each individual voxel. 
     vox_tri_samps = sparse.dok_matrix((spc.size.prod(), 
-        in_surf.tris.shape[0]), dtype=NP_FLOAT)
-    sampler = np.linspace(0, 1, 2*factor + 1, dtype=NP_FLOAT)[1:-1:2]
-    samples = (np.stack(np.meshgrid(sampler, sampler, sampler), axis=-1)
+        tris.shape[0]), dtype=NP_FLOAT)
+    samplers = [ np.linspace(0, 1, 2*f + 1, dtype=NP_FLOAT)[1:-1:2] 
+                    for f in factor ]
+    samples = (np.stack(np.meshgrid(*samplers), axis=-1)
                .reshape(-1,3) - 0.5)
 
     for t in t_range: 
@@ -918,7 +959,7 @@ def _vox_tri_weights_worker(t_range, in_surf, out_surf, spc, factor, ones=False)
         # vertices (note, NOT A SHUFFLE) such that the highest index is first 
         # (corresponding to A,a). The relative ordering of vertices remains the
         # same, so we use flagsum to check if B < C or C < B. 
-        tri = in_surf.tris[t,:]
+        tri = tris[t,:]
         tri_max = np.argmax(tri)
         tri_sort = [ tri[(tri_max + i) % 3] for i in range(3) ]
         flagsum = sum([ int(tri_sort[v] < tri_sort[(v + 1) % 3]) 
@@ -932,22 +973,26 @@ def _vox_tri_weights_worker(t_range, in_surf, out_surf, spc, factor, ones=False)
         else:
             tets = TETRA2
 
-        hull_ps = np.vstack((in_surf.points[tri_sort,:], 
-                             out_surf.points[tri_sort,:]))
+        hull_ps = np.vstack((inps_vox[tri_sort,:], 
+                             outps_vox[tri_sort,:]))
 
         # Get the neighbourhood of voxels through which this prism passes
-        # in linear indices
+        # in linear indices (note the +1 on the upper bound)
         bbox = (np.vstack((np.maximum(0, hull_ps.min(0)),
                            np.minimum(spc.size, hull_ps.max(0)+1)))
                            .round().astype(np.int32))
-        hood = (np.stack(np.meshgrid(
-                *[ range(*bbox[:,d]) for d in range(3) ]), axis=-1)
-                .reshape(-1,3).astype(np.int32))             
+        hood = np.array(list(itertools.product(
+                range(*bbox[:,0]), range(*bbox[:,1]), range(*bbox[:,2])
+                )), dtype=np.int32)
+
+        # The bbox may not intersect any voxels within the FoV at all, skip
+        if not hood.size:
+            continue 
         hood_vidx = np.ravel_multi_index(hood.T, spc.size)
 
         # Debug mode: just stick ones in all candidate voxels and continue 
         if ones: 
-            vox_tri_samps[hood_vidx,t] = factor ** 3
+            vox_tri_samps[hood_vidx,t] = factor.prod()
             continue
 
         for vidx, ijk in zip(hood_vidx, hood.astype(NP_FLOAT)):
@@ -993,7 +1038,7 @@ def _vox_tri_weights_worker(t_range, in_surf, out_surf, spc, factor, ones=False)
             # double counting within the polyhedra (although in theory this 
             # shouldn't happen). Hull formation can fail due to geometric 
             # degeneracy so wrap it up in a try block 
-            samps_in = np.zeros(v_samps.shape[0], dtype=np.bool)
+            samps_in = np.zeros(v_samps.shape[0], dtype=bool)
             for tet in tets: 
                 try: 
                     hull = Delaunay(hull_ps[tet,:])
@@ -1117,7 +1162,7 @@ def vtx_tri_weights(surf, cores=mp.cpu_count()):
                                     edges, edge_lengths)
 
     if cores > 1: 
-        worker_lists = utils._distributeObjects(range(points.shape[0]), cores)
+        worker_lists = utils._distributeObjects(range(surf.n_points), cores)
         with mp.Pool(cores) as p: 
             results = p.map(worker_func, worker_lists)
 
@@ -1127,7 +1172,7 @@ def vtx_tri_weights(surf, cores=mp.cpu_count()):
             vtx_tri_weights += r 
 
     else: 
-        vtx_tri_weights = worker_func(range(points.shape[0]))
+        vtx_tri_weights = worker_func(range(surf.n_points))
 
     assert (vtx_tri_weights.data > 0).all(), 'Zero areas returned'
     return vtx_tri_weights 
