@@ -22,7 +22,7 @@ def cortex(ref, struct2ref, **kwargs):
     Required args: 
         ref (str/regtricks ImageSpace): voxel grid in which to estimate PVs. 
         struct2ref (str/np.array/rt.Registration): registration between space 
-            of surface and reference. Use 'I' for identity. 
+            of surface and reference (see -flirt and -stuct). Use 'I' for identity. 
         fsdir (str): path to a FreeSurfer subject directory. 
         LWS/LPS/RWS/RPS (str): individual paths to the surfaces,
             eg LWS = Left White surface, RPS = Right Pial surace. 
@@ -35,7 +35,7 @@ def cortex(ref, struct2ref, **kwargs):
  
     Returns: 
         (np.array), 4D, size equal to the reference image, with the PVs arranged 
-            GM/WM/non-brain in 4th dim. 
+            GM/WM/non-brain in 4th dim.
     """
 
     if not any([
@@ -45,26 +45,7 @@ def cortex(ref, struct2ref, **kwargs):
         raise RuntimeError("Either a fsdir or paths to LWS/LPS etc"
             "must be given.")
 
-    # If subdir given, then get all the surfaces out of the surf dir
-    # If individual surface paths were given they will already be in scope
-    if kwargs.get('fsdir'):
-        surfdict = utils._loadSurfsToDict(kwargs['fsdir'])
-        kwargs.update(surfdict)
-
-    # What hemispheres are we working with?
-    sides = []
-    if np.all([ (kwargs.get(s) is not None) for s in ['LPS', 'LWS'] ]): 
-        sides.append('L')
-
-    if np.all([ kwargs.get(s) is not None for s in ['RPS', 'RWS'] ]): 
-        sides.append('R')
-
-    if not sides:
-        raise RuntimeError("At least one hemisphere (eg LWS/LPS required")
-
-    # Load reference ImageSpace object
-    hemispheres = [ Hemisphere(kwargs[s+'WS'], kwargs[s+'PS'], s) 
-        for s in sides ] 
+    hemispheres = utils.load_surfs_to_hemispheres(**kwargs)
 
     # Either create local copy of ImageSpace object or init from path 
     if isinstance(ref, ImageSpace):
@@ -74,8 +55,8 @@ def cortex(ref, struct2ref, **kwargs):
 
     # Set supersampler and estimate. 
     if kwargs.get('supersample') is None:
-        supersampler = np.ceil(ref_space.vox_size.round(1) 
-                                / 0.75).astype(np.int8)
+        supersampler = np.maximum(np.floor(ref_space.vox_size.round(1) 
+                                / 0.75), 1).astype(np.int32)
     else: 
         supersampler = kwargs.get('supersample') * np.ones(3)
 
@@ -93,13 +74,13 @@ def structure(ref, struct2ref, **kwargs):
     Required args: 
         ref (str/regtricks ImageSpace): voxel grid in which to estimate PVs. 
         struct2ref (str/np.array/rt.Registration): registration between space 
-            of surface and reference. Use 'I' for identity. 
-        surf (str): path to surface (see space argument below)
+            of surface and reference (see -flirt and -stuct). Use 'I' for identity. 
+        surf (str): path to surface (see coords argument below)
 
     Optional args: 
         flirt (bool): denoting struct2ref is FLIRT transform; if so, set struct. 
-        space (str): space in which surface is defined: default is 'world' (mm coords),
-            for FIRST surfaces set as 'first' and provide struct argument 
+        coords (str): convention by which surface is defined: default is 'world' 
+            (mm coords), for FIRST surfaces set as 'fsl' and provide struct argument 
         struct (str): path to structural image from which surfaces were derived
         cores (int): number of cores to use, default 8 
         supersample (int/array): single or 3 values, supersampling factor
@@ -112,17 +93,16 @@ def structure(ref, struct2ref, **kwargs):
     if not bool(kwargs.get('surf')):
         raise RuntimeError("surf kwarg must be a Surface object or path to one")
 
-    if not kwargs.get('space'): 
-        kwargs['space'] = 'world'
-
-    if kwargs['space'] == 'first' and not kwargs.get('struct'):
+    coords = kwargs.get('coords', 'world')
+    if coords == 'fsl' and not kwargs.get('struct'):
         raise RuntimeError("Structural image must be supplied for FIRST surfs")
 
     if type(kwargs['surf']) is str: 
+        surf = Surface(kwargs['surf'], name=op.split(kwargs['surf'])[1])
+        if kwargs.get('coords', 'world') == 'fsl':
+            struct_spc = ImageSpace(kwargs['struct'])
+            surf = surf.transform(struct_spc.FSL2world)
 
-        surf = Surface(kwargs['surf'], kwargs['space'], kwargs.get('struct'), 
-            op.split(kwargs['surf'])[1])
-    
     elif type(kwargs['surf']) is not Surface: 
         raise RuntimeError("surf kwarg must be a Surface object or path to one")
 
@@ -136,8 +116,8 @@ def structure(ref, struct2ref, **kwargs):
         ref_space = ImageSpace(ref)
 
     if kwargs.get('supersample') is None:
-        supersampler = np.ceil(ref_space.vox_size.round(1) 
-                                / 0.75).astype(np.int8)
+        supersampler = np.maximum(np.floor(ref_space.vox_size.round(1) 
+                                / 0.75), 1).astype(np.int32)
     else: 
         supersampler = kwargs.get('supersample') * np.ones(3)
 
@@ -162,21 +142,18 @@ def complete(ref, struct2ref, **kwargs):
     Required args: 
         ref (str/regtricks ImageSpace): voxel grid in which to estimate PVs. 
         struct2ref (str/np.array/rt.Registration): registration between space 
-            of surface and reference. Use 'I' for identity. 
-        anat: path to augmented fsl_anat directory (see -fsl_fs_anat command).
-            This REPLACES fsdir, firstdir, fastdir, LPS/RPS etc args 
-
-    Alternatvies to anat argument (all required): 
-        fsdir (str): FreeSurfer subject directory, OR: 
-        LWS/LPS/RWS/RPS (str): paths to individual surfaces (L/R white/pial)
+            of surface and reference (see -flirt and -stuct). Use 'I' for identity. 
+        fslanat: path to fslanat directory. This REPLACES firstdir/fastdir/struct. 
         firstdir (str): FIRST directory in which .vtk surfaces are located
         fastdir (str): FAST directory in which _pve_0/1/2 are located 
-        struct (str): path to structural image from which surfaces were dervied
+        struct (str): path to structural image from which FIRST surfaces were dervied
+        fsdir (str): FreeSurfer subject directory, OR: 
+        LWS/LPS/RWS/RPS (str): paths to individual surfaces (L/R white/pial)
 
     Optional args: 
         flirt (bool): denoting struct2ref is FLIRT transform; if so, set struct. 
-        space (str): space in which surface is defined: default is 'world' (mm coords),
-            for FIRST surfaces set as 'first' and provide struct argument 
+        coords (str): convention by which surface is defined: default is 'world' 
+            (mm coords), for FIRST surfaces set as 'fsl' and provide struct argument 
         struct (str): path to structural image from which surfaces were derived
         cores (int): number of cores to use, default 8 
         supersample (int/array): single or 3 values, supersampling factor
@@ -190,12 +167,12 @@ def complete(ref, struct2ref, **kwargs):
 
     # If anat dir then various subdirs are loaded by @enforce_common_args
     # If not then direct load below 
-    if not bool(kwargs.get('anat')):
-        if not bool(kwargs.get('fsdir')):
-            if not all([ bool(kwargs.get(k)) for k in ['LWS','LPS','RWS','RPS'] ]):
-                raise RuntimeError("If fsdir not given, " + 
-                    "provide paths for LWS,LPS,RWS,RPS")
-        
+    if not bool(kwargs.get('fsdir')):
+        if not all([ bool(kwargs.get(k)) for k in ['LWS','LPS','RWS','RPS'] ]):
+            raise RuntimeError("If fsdir not given, " + 
+                "provide paths for LWS,LPS,RWS,RPS")
+
+    if not bool(kwargs.get('fslanat')):
         if not (bool(kwargs.get('fastdir')) and bool(kwargs.get('firstdir'))):
             raise RuntimeError("If not using anat dir, fastdir/firstdir required")
    
@@ -211,26 +188,31 @@ def complete(ref, struct2ref, **kwargs):
         
     # Process subcortical structures first. 
     FIRSTsurfs = utils._loadFIRSTdir(kwargs['firstdir'])
-    structures = [ Surface(surf, 'first', kwargs['struct'], name) 
-        for name, surf in FIRSTsurfs.items() ]
-    print("Structures found: ", end=' ')
-    [ print(s.name, end=' ') for s in structures ]
-    print('Cortex')
-    desc = 'Subcortical structures'
+    subcortical = []
+    struct_spc = ImageSpace(kwargs['struct'])
+    for name, surf in FIRSTsurfs.items(): 
+        s = Surface(surf, name)
+        s = s.transform(struct_spc.FSL2world)
+        subcortical.append(s)
+    
+    disp = "Structures found: " + ", ".join([ 
+        s.name for s in subcortical ] + ['Cortex'])
+    print(disp)
 
     # To estimate against each subcortical structure, we apply the following
     # partial func to each using a map() call. Carry kwargs from this func 
+    desc = 'Subcortical structures'
     estimator = functools.partial(__structure_wrapper, 
                                   ref=ref, struct2ref=struct2ref, **kwargs)
 
-    # This is equivalent to a map(estimator, structures) call
+    # This is equivalent to a map(estimator, subcortical) call
     # All the extra stuff (tqdm etc) is used for progress bar
     results = [ pv for _, pv in 
-        tqdm.tqdm(enumerate(map(estimator, structures)), 
-        total=len(structures), desc=desc, bar_format=core.BAR_FORMAT, 
+        tqdm.tqdm(enumerate(map(estimator, subcortical)), 
+        total=len(subcortical), desc=desc, bar_format=core.BAR_FORMAT, 
         ascii=True) ] 
 
-    output.update(dict(zip([s.name for s in structures], results)))
+    output.update(dict(zip([s.name for s in subcortical], results)))
 
     # Now do the cortex, then stack the whole lot 
     ctx  = cortex(ref=ref, struct2ref=struct2ref, **kwargs)
