@@ -31,11 +31,14 @@ def _cortex(hemispheres, space, struct2ref, supersampler, cores, ones):
 
     if not isinstance(loc_hemispheres, list):
         loc_hemispheres = [loc_hemispheres]
-        
-    surfs = [ s for h in loc_hemispheres for s in h.surfs ]
 
+    for h in loc_hemispheres: 
+        h.inSurf, h.outSurf = [ s.transform(struct2ref) for s in h.surfs ]
+
+    surfs = [ s for h in loc_hemispheres for s in h.surfs ]
     for s in surfs: 
-        s.index_on(space, struct2ref, cores)
+        s.index_on(space, cores)
+        s.indexed.voxelised = s.voxelise(space, cores)
 
     # Estimate PV fractions for each surface
     for h in loc_hemispheres:
@@ -58,7 +61,7 @@ def _cortex(hemispheres, space, struct2ref, supersampler, cores, ones):
         hemiPVs = np.zeros((np.prod(space.size), 3), dtype=NP_FLOAT)
         hemiPVs[:,1] = in_pvs 
         hemiPVs[:,0] = np.maximum(0.0, out_pvs - in_pvs)
-        hemiPVs[:,2] = 1.0 - np.sum(hemiPVs[:,0:2], axis=1)
+        hemiPVs[:,2] = 1.0 - (hemiPVs[:,0:2].sum(1))
         h.PVs = hemiPVs
 
     # Merge the hemispheres, giving priority to GM, then WM, then CSF.
@@ -72,7 +75,7 @@ def _cortex(hemispheres, space, struct2ref, supersampler, cores, ones):
         outPVs[:,0] = np.minimum(1.0, h1.PVs[:,0] + h2.PVs[:,0])
         outPVs[:,1] = np.minimum(1.0 - outPVs[:,0],
             h1.PVs[:,1] + h2.PVs[:,1])
-        outPVs[:,2] = 1.0 - np.sum(outPVs[:,0:2], axis=1)
+        outPVs[:,2] = 1.0 - outPVs[:,0:2].sum(1)
 
     # Sanity checks
     if np.any(outPVs > 1.0): 
@@ -108,7 +111,9 @@ def _structure(surf, space, struct2ref, supersampler, ones, cores):
 
     # Create our own local copy of inputs 
     loc_surf = copy.deepcopy(surf)
-    loc_surf.index_on(space, struct2ref, cores)
+    loc_surf = loc_surf.transform(struct2ref)
+    loc_surf.index_on(space, cores)
+    loc_surf.indexed.voxelised = loc_surf.voxelise(space, cores)
     loc_surf._estimate_fractions(supersampler, cores, ones)
     
     return loc_surf.output_pvs(space)
