@@ -13,7 +13,7 @@ import copy
 import numpy as np 
 from fsl.wrappers import fsl_anat
 import regtricks as rt
-from scipy import sparse
+from scipy.sparse.linalg import eigs
 
 NP_FLOAT = np.float32
 
@@ -46,10 +46,9 @@ def _mp_call_attribute(obj, method_name, args=None):
 
 
 def check_anat_dir(dir):
-    """Check that dir contains output from FIRST, FAST and FreeSurfer"""
+    """Check that dir contains output from FIRST and FAST"""
 
     return all([
-        op.isdir(op.join(dir, 'fs', 'surf')), 
         op.isdir(op.join(dir, 'first_results')), 
         op.isfile(op.join(dir, 'T1_fast_pve_0.nii.gz')), 
     ])
@@ -231,7 +230,7 @@ def _runFAST(struct, dir):
     os.chdir(pwd)
 
 
-def affineTransformPoints(points, affine):
+def affine_transform(points, affine):
     """Apply affine transformation to set of points.
 
     Args: 
@@ -255,16 +254,6 @@ def affineTransformPoints(points, affine):
     return np.squeeze(transfd[0:3,:].T)
 
 
-def _coordinatesForGrid(ofSize):
-    """Produce N x 3 array of all voxel indices (eg [10, 18, 2]) within
-    a grid of size ofSize, 0-indexed and in integer form. 
-    """
-
-    I, J, K = np.unravel_index(np.arange(np.prod(ofSize)), ofSize)
-    cents = np.vstack((I.flatten(), J.flatten(), K.flatten())).T
-    return cents.astype(np.int32)
-
-
 def _distributeObjects(objs, ngroups):
     """Distribute a set of objects into n groups.
     For preparing chunks before multiprocessing.pool.map
@@ -288,74 +277,6 @@ def _distributeObjects(objs, ngroups):
     return chunks 
 
 
-def _clipArray(arr, mini=0.0, maxi=1.0):
-    """Clip array values into range [mini, maxi], default [0 1]"""
-
-    arr[arr < mini] = mini 
-    arr[arr > maxi] = maxi 
-    return arr 
-
-
-def fsl_fs_anat(**kwargs):
-    """
-    Run fsl_anat (FAST & FIRST) and augment output with FreeSurfer
-
-    Args: 
-        anat: (optional) path to existing fsl_anat dir to augment
-        struct: (optional) path to T1 NIFTI to create a fresh fsl_anat dir
-        out: output path (default alongside input, named input.anat)
-    """
-
-    # We are either adding to an existing dir, or we are creating 
-    # a fresh one 
-    if (not bool(kwargs.get('anat'))) and (not bool(kwargs.get('struct'))):
-        raise RuntimeError("Either a structural image or a path to an " + 
-            "existing fsl_anat dir must be given")
-
-    if kwargs.get('struct') and (not op.isfile(kwargs['struct'])):
-        raise RuntimeError("No struct image given, or does not exist")
-
-    if kwargs.get('anat') and (not op.isdir(kwargs['anat'])):
-        raise RuntimeError("fsl_anat dir does not exist")
-
-    debug = bool(kwargs.get('debug'))
-    anat_exists = bool(kwargs.get('anat'))
-    struct = kwargs['struct']
-
-    # Run fsl_anat if needed. Either use user-supplied name or default
-    if not anat_exists:
-        outname = kwargs.get('out')
-        if not outname:
-            outname = _splitExts(kwargs['struct'])[0]
-            outname = op.dirname(kwargs['struct']) + outname
-        print("Preparing an fsl_anat dir at %s" % outname)
-        if outname.endswith('.anat'):
-            outname = outname[:-5]
-        fsl_anat(struct, outname)
-        outname += '.anat'
-
-    else:
-        outname = kwargs['anat']
-    
-    # Run the surface steps if reqd. 
-    # Check the fullfov T1 exists within anat_dir
-    if not op.isdir(op.join(outname, 'fs', 'surf')):
-        fullfov = op.join(outname, 'T1_fullfov.nii.gz')
-
-        if not op.isfile(fullfov):
-            raise RuntimeError("Could not find T1_fullfov.nii.gz within anat_dir %s" 
-                    % outname)
-
-        print("Adding FreeSurfer to fsl_anat dir at %s" % outname)
-        _runFreeSurfer(fullfov, outname, debug)
-
-    if not check_anat_dir(outname): 
-        raise RuntimeError("fsl_anat dir should be complete with surfaces") 
-
-    print("fsl_anat dir at %s is now complete with surfaces" % outname)
-    return outname 
-
-
 @cascade_attributes
 def enforce_and_load_common_arguments(func):
     """
@@ -372,7 +293,7 @@ def enforce_and_load_common_arguments(func):
             If given as 'I', identity matrix will be used. 
 
     Optional args: 
-        anat: a fsl_anat directory (created/augmented by make_surf_anat_dir)
+        fslanat: a fslanat directory
         flirt: bool denoting that the struct2ref is a FLIRT transform.
             This means it requires special treatment. If set, then it will be
             pre-processed in place by this function, and then the flag will 
@@ -392,32 +313,31 @@ def enforce_and_load_common_arguments(func):
         if (not isinstance(ref, rt.ImageSpace)): 
             ref = rt.ImageSpace(ref)
 
-        # If given a anat_dir we can load the structural image in 
-        if kwargs.get('anat'):
-            if not check_anat_dir(kwargs['anat']):
-                raise RuntimeError("anat is not complete: it must contain " 
-                    "fast, fs and first subdirectories")
+        # If given a fslanat dir we can load the structural image in 
+        if kwargs.get('fslanat'):
+            if not check_anat_dir(kwargs['fslanat']):
+                raise RuntimeError("fslanat is not complete: it must contain " 
+                    "FAST output and a first_results subdirectory")
 
-            kwargs['fastdir'] = kwargs['anat']
-            kwargs['fsdir'] = op.join(kwargs['anat'], 'fs')
-            kwargs['firstdir'] = op.join(kwargs['anat'], 'first_results')
+            kwargs['fastdir'] = kwargs['fslanat']
+            kwargs['firstdir'] = op.join(kwargs['fslanat'], 'first_results')
 
             # If no struct image given, try and pull it out from the anat dir
             # But, if it has been cropped relative to original T1, then give
             # warning (as we will not be able to convert FLIRT to world-world)
             if not kwargs.get('struct'): 
                 if kwargs.get('flirt'):
-                    matpath = glob.glob(op.join(kwargs['anat'], '*nonroi2roi.mat'))[0]
+                    matpath = glob.glob(op.join(kwargs['fslanat'], '*nonroi2roi.mat'))[0]
                     nonroi2roi = np.loadtxt(matpath)
                     if np.any(np.abs(nonroi2roi[0:3,3])):
                         print("Warning: T1 was cropped relative to T1_orig within" + 
-                            " fsl_fs_anat dir.\n Please ensure the struct2ref FLIRT" +
+                            " fslanat dir.\n Please ensure the struct2ref FLIRT" +
                             " matrix is referenced to T1, not T1_orig.")
 
-                s = op.join(kwargs['anat'], 'T1.nii.gz')
+                s = op.join(kwargs['fslanat'], 'T1.nii.gz')
                 kwargs['struct'] = s
                 if not op.isfile(s):
-                    raise RuntimeError("Could not find T1.nii.gz in the anat dir")
+                    raise RuntimeError("Could not find T1.nii.gz in the fslanat dir")
 
  
         # Structural to reference transformation. Either as array, path
@@ -507,11 +427,11 @@ def sparse_normalise(mat, axis, threshold=1e-6):
 
     Args: 
         mat: sparse matrix to normalise 
-        axis: dimension along which sums should equal 1 (0 for col, 1 for row)
-        threshold: any row/col wuth sum < threshold will be set to zero  
+        axis: dimension for which sum should equal 1 (1 for row, 0 for col)
+        threshold: any row/col with sum < threshold will be set to zero  
 
     Returns: 
-        sparse matrix. either CSR (axis 0) or CSC (axis 1)
+        sparse matrix, same format as input. 
     """
 
     # Make local copy - otherwise this function will modify the caller's copy 
@@ -520,14 +440,13 @@ def sparse_normalise(mat, axis, threshold=1e-6):
 
     if axis == 0:
         matrix = mat.tocsr()
-        norm = mat.sum(0).A.flatten()
     elif axis == 1: 
         matrix = mat.tocsc()
-        norm = mat.sum(1).A.flatten()
     else: 
-        raise RuntimeError("Axis must be 0 or 1")
+        raise ValueError("Axis must be 0 or 1")
 
     # Set threshold. Round any row/col below this to zeros 
+    norm = mat.sum(axis).A.flatten()
     fltr = (norm > threshold)
     normalise = np.zeros(norm.size)
     normalise[fltr] = 1 / norm[fltr]
@@ -535,12 +454,130 @@ def sparse_normalise(mat, axis, threshold=1e-6):
 
     # Sanity check
     sums = matrix.sum(axis).A.flatten()
-    assert np.all(np.abs((sums[sums > 0] - 1)) < 1e-6), 'Did not normalise to 1'
+    assert np.abs((sums[sums > 0] - 1)).max() < (1e2 * threshold), 'Did not normalise to 1'
     return constructor(matrix)
 
 
 def is_symmetric(a, tol=1e-9): 
     return not (np.abs(a - a.T) > tol).max()
 
+
 def is_nsd(a):
-    return not (sparse.linalg.eigs(a)[0] > 0).any()
+    return not (eigs(a)[0] > 0).any()
+
+
+def calc_midsurf(in_surf, out_surf):
+    """
+    Midsurface between two Surfaces
+    """
+    from .classes import Surface
+    vec = out_surf.points - in_surf.points 
+    points =  in_surf.points + (0.5 * vec)
+    return Surface.manual(points, in_surf.tris)
+
+
+def calculateXprods(points, tris):
+    """
+    Normal vectors for points,triangles array. 
+    For triangle vertices ABC, this is calculated as (C - A) x (B - A). 
+    """
+
+    return np.cross(
+        points[tris[:,2],:] - points[tris[:,0],:], 
+        points[tris[:,1],:] - points[tris[:,0],:], 
+        axis=1)
+
+
+def slice_sparse(mat, slice0, slice1):
+    """
+    Slice a block out of a sparse matrix, ie mat[slice0,slice1]. 
+    Scipy sparse matrices do not support slicing in this manner (unlike numpy)
+
+    Args: 
+        mat (sparse): of any form 
+        slice0 (bool,array): mask to apply on axis 0 (rows)
+        slice1 (bool,array): mask to apply on axis 1 (columns)
+
+    Returns: 
+        CSR matrix
+    """
+    
+    out = mat.tocsc()[:,slice1]
+    return out.tocsr()[slice0,:]
+
+
+def rebase_triangles(points, tris, tri_inds):
+    """
+    Re-express a patch of a larger surface as a new points and triangle
+    matrix pair, indexed from 0. Useful for reducing computational 
+    complexity when working with a small patch of a surface where only 
+    a few nodes in the points array are required by the triangles matrix. 
+
+    Args: 
+        points (np.array): surface vertices, P x 3
+        tris (np.array): surface triangles, T x 3
+        tri_inds (np.array): row indices into triangles array, to rebase
+    
+    Returns: 
+        (points, tris) tuple of re-indexed points/tris. 
+    """
+
+    ps = np.empty((0, 3), dtype=NP_FLOAT)
+    ts = np.empty((len(tri_inds), 3), dtype=np.int32)
+    pointsLUT = []
+
+    for t in range(len(tri_inds)):
+        for v in range(3):
+
+            # For each vertex of each tri, check if we
+            # have already processed it in the LUT
+            vtx = tris[tri_inds[t],v]
+            idx = np.argwhere(pointsLUT == vtx)
+
+            # If not in the LUT, then add it and record that
+            # as the new position. Write the missing vertex
+            # into the local points array
+            if not idx.size:
+                pointsLUT.append(vtx)
+                idx = len(pointsLUT) - 1
+                ps = np.vstack([ps, points[vtx,:]])
+
+            # Update the local triangle
+            ts[t,v] = idx
+
+    return (ps, ts)
+
+
+def space_encloses_surface(space, points_vox):
+
+    if np.round(np.min(points_vox)) < 0: 
+        return False 
+    if (np.round(np.max(points_vox, axis=0)) >= space.size).any(): 
+        return False 
+    return True 
+
+
+def load_surfs_to_hemispheres(**kwargs):
+
+    from .classes import Hemisphere
+    # If subdir given, then get all the surfaces out of the surf dir
+    # If individual surface paths were given they will already be in scope
+    if kwargs.get('fsdir'):
+        surfdict = _loadSurfsToDict(kwargs['fsdir'])
+        kwargs.update(surfdict)
+
+    # What hemispheres are we working with?
+    sides = []
+    if np.all([ (kwargs.get(s) is not None) for s in ['LPS', 'LWS'] ]): 
+        sides.append('L')
+
+    if np.all([ kwargs.get(s) is not None for s in ['RPS', 'RWS'] ]): 
+        sides.append('R')
+
+    if not sides:
+        raise RuntimeError("At least one hemisphere (eg LWS/LPS) required")
+
+    hemispheres = [ Hemisphere(kwargs[s+'WS'], kwargs[s+'PS'], s) 
+        for s in sides ] 
+
+    return hemispheres
\ No newline at end of file
